/* 
 * packet sending and receiving routines for ncovert
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <linux/ip.h>
#include <linux/tcp.h>
#include <signal.h>
#include "ncovert.h"

void send_packets(unsigned int source_addr, unsigned int dest_addr, unsigned short source_port, unsigned short dest_port, char *filename, int quiet) 
{
	struct send_tcp
	{
		struct iphdr ip;
		struct tcphdr tcp;
	} send_tcp;
	
	struct pseudo_header
	{
		unsigned int source_address;
		unsigned int dest_address;
		unsigned char placeholder;
		unsigned char protocol;
		unsigned short tcp_length;
		struct tcphdr tcp;
	} pseudo_header;
	
	int send_socket;
	struct sockaddr_in sai;
	FILE *input;
	blocks block;
	int size=0;
	
	srand((getpid())*(dest_port)); 
	
	if((input=fopen(filename,"rb"))== NULL)
	{
		printf("I cannot open the file %s for reading\n",filename);
		exit(1);
	}
	else while(1)
	{
		guaranteed_memset(block.byte_chunk,0,4);
		size=fread(block.byte_chunk,1,4,input);
		if (size <= 0) break; 
		sleep(1);
		send_tcp.ip.ihl = 5;
		send_tcp.ip.version = 4;
		send_tcp.ip.tos = 0;
		send_tcp.ip.tot_len = htons(40);
		send_tcp.ip.id =(int)(255.0*rand()/(RAND_MAX+1.0)); 
		send_tcp.ip.frag_off = 0;
		send_tcp.ip.ttl = 64; 
		send_tcp.ip.protocol = IPPROTO_TCP;
		send_tcp.ip.check = 0;
		send_tcp.ip.saddr = source_addr;
		send_tcp.ip.daddr = dest_addr;
		if(source_port == 0)
			send_tcp.tcp.source = 1+(int)(10000.0*rand()/(RAND_MAX+1.0));
		else
			send_tcp.tcp.source = htons(source_port); 
		send_tcp.tcp.seq = block.long_chunk;
		send_tcp.tcp.dest = htons(dest_port);
		send_tcp.tcp.ack_seq = 0;
		send_tcp.tcp.res1 = 0;
		send_tcp.tcp.doff = 5;
		send_tcp.tcp.fin = 0;
		send_tcp.tcp.syn = 1;
		send_tcp.tcp.rst = 0;
		send_tcp.tcp.psh = 0;
		send_tcp.tcp.ack = 0;
		send_tcp.tcp.urg = 0;
		send_tcp.tcp.window = htons(512);
		send_tcp.tcp.check = 0;
		send_tcp.tcp.urg_ptr = 0;
		
		sai.sin_family = AF_INET;
		sai.sin_port = send_tcp.tcp.source;
		sai.sin_addr.s_addr = send_tcp.ip.daddr;   
		
		send_socket = socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
		if(send_socket < 0)
		{
			perror("Unable to open socket");
			exit(1);
		}
		
		send_tcp.ip.check = in_cksum((unsigned short *)&send_tcp.ip, 20);
		pseudo_header.source_address = send_tcp.ip.saddr;
		pseudo_header.dest_address = send_tcp.ip.daddr;
		pseudo_header.placeholder = 0;
		pseudo_header.protocol = IPPROTO_TCP;
		pseudo_header.tcp_length = htons(20);
		bcopy((char *)&send_tcp.tcp, (char *)&pseudo_header.tcp, 20);
		send_tcp.tcp.check = in_cksum((unsigned short *)&pseudo_header, 32);
		sendto(send_socket, &send_tcp, 40, 0, (struct sockaddr *)&sai, sizeof(sai));
		if (verbose) printf("Sending Data: 0x%08x\n",block.long_chunk);
		else if (!quiet) printf(".");
		close(send_socket);
	}
	fclose(input);
}

void recv_packets(unsigned int source_addr, unsigned int dest_addr, unsigned short source_port, unsigned short dest_port, char *filename, int ack, int quiet)
{
	  struct recv_tcp
	  {
		  struct iphdr ip;
		  struct tcphdr tcp;
		  char buffer[10000];
	  } recv_pkt;

	  int recv_socket;
	  FILE *output;
	  blocks block;
	  
	  if((output=fopen(filename,"wb"))== NULL)
	  {
		  printf("Can't open file %s\n",filename);
		  exit(1);
	  }
	  signal(SIGALRM,caughtsig);
	  alarm(timeout);
	  while(1)
	  {
		  recv_socket = socket(AF_INET, SOCK_RAW, 6);
		  if(recv_socket < 0)
		  {
			  perror("Unable to open socket");
			  exit(1);
		  }
		  read(recv_socket, (struct recv_tcp *)&recv_pkt, 9999);
		  if (source_port == 0)
		  {
			  if((recv_pkt.tcp.syn == 1) && (recv_pkt.ip.saddr == source_addr))
			  {
				  if (ack==0)
				  {
					  if (verbose) printf("Receiving Data: %x\n",recv_pkt.tcp.seq);
					  else if (!quiet) printf(".");
					  block.long_chunk=recv_pkt.tcp.seq;
					  alarm(timeout);
					  fwrite(block.byte_chunk,4,1,output); 
					  fflush(output);
				  }
				  else
				  {
					  if (verbose) printf("Receiving Data: %x\n",recv_pkt.tcp.ack_seq);
					  else if (!quiet) printf(".");
					  block.long_chunk=recv_pkt.tcp.ack_seq-1;
					  alarm(timeout);
					  fwrite(block.byte_chunk,4,1,output); 
					  fflush(output);
				  }
			  }
		  }
		  else
		  {
			  if((recv_pkt.tcp.syn==1) && (ntohs(recv_pkt.tcp.source) == source_port))
			  {
				  if (ack==0)
				  {
					  if (verbose) printf("Receiving Data: %x\n",recv_pkt.tcp.seq);
					  else if (!quiet) printf(".");
					  block.long_chunk=recv_pkt.tcp.seq;
					  alarm(timeout);
					  fwrite(block.byte_chunk,4,1,output); 
					  fflush(output);
				  }
				  else
				  {
					  if (verbose) printf("Receiving Data: %x\n",recv_pkt.tcp.ack_seq);
					  else if (!quiet) printf(".");
					  block.long_chunk=recv_pkt.tcp.ack_seq-1;
					  alarm(timeout);
					  fwrite(block.byte_chunk,4,1,output); 
					  fflush(output);
				  }
			  }
		  }
		  close(recv_socket);
	  }
	  fclose(output);
}
