/* 
 * util.c - utility routines for ncovert
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <unistd.h>
#include "ncovert.h"

/*
 * true working memset as compiler could 'optimize' out the regular memset
 */
void *guaranteed_memset(void *v, int c, size_t n)
{
	volatile char *p=v;
	while (n--) *p++=c;
	return v;
}

/*
 * simple address lookup
 */
unsigned int host_convert(char *hostname)
{
	static struct in_addr ha;
	struct hostent *he;
	ha.s_addr = inet_addr(hostname);
	if(ha.s_addr == -1)
	{
		he = gethostbyname(hostname);
		if(he == NULL)
		{
			fprintf(stderr, "cannot resolve %s\n", hostname);
			exit(0);
		}
		bcopy(he->h_addr, (char *)&ha.s_addr, he->h_length);
	}
	return ha.s_addr;
}

/*
 * handle alarm
 */
void caughtsig(int sig)
{
	if (verbose) printf("Timed out\n");
	exit(sig);
}

/* 
 * generic cksum routine is from old ping.c, note the copyright
 */
/* Copyright (c)1987 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * dupliated in all such forms and that any documentation, advertising 
 * materials, and other materials related to such distribution and use
 * acknowledge that the software was developed by the University of
 * California, Berkeley. The name of the University may not be used
 * to endorse or promote products derived from this software without
 * specific prior written permission. THIS SOFTWARE IS PROVIDED ``AS
 * IS'' AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, 
 * WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHATIBILITY AND 
 * FITNESS FOR A PARTICULAR PURPOSE
 */
unsigned short in_cksum(unsigned short *ptr, int nbytes)
{
  register long           sum;
  u_short                 oddbyte;
  register u_short        answer;

  sum = 0;
  while (nbytes > 1) {sum += *ptr++; nbytes -= 2;}
  if (nbytes == 1) {oddbyte = 0; *((u_char *) &oddbyte) = *(u_char *)ptr; sum += oddbyte;}
  sum  = (sum >> 16) + (sum & 0xffff);
  sum += (sum >> 16);                     
  answer = ~sum;
  return(answer);
}


